@@ -25,9 +25,8 @@ logger.log_and_write(f, rect)
 pt = parser.parse(f)
 assert pt.x == pt.y == 1
 
-logger.log_and_write(f, PointGroup(count=10))
-for i in range(10):
-    logger.log_and_write(f, rand_point())
+points = [rand_point() for i in range(10)]
+logger.log_and_write(f, PointGroup(count=10, points=points))
 rect = parser.parse(f)
 assert rect.code == Rectangle.code.always
 
