@@ -17,11 +17,10 @@ class Handler(TCPHandler):
     
     def point_group(self, pg):
         """returns a rectangle which encompasses all points in the group"""
-        points = [Point.parse(self.rfile) for i in range(pg.count)]
-        xmin = min(p.x for p in points)
-        xmax = max(p.x for p in points)
-        ymin = min(p.y for p in points)
-        ymax = max(p.y for p in points)
+        xmin = min(p.x for p in pg.points)
+        xmax = max(p.x for p in pg.points)
+        ymin = min(p.y for p in pg.points)
+        ymax = max(p.y for p in pg.points)
         return Rectangle(points=[
             Point(x=xmin, y=ymin), Point(x=xmin, y=ymax),
             Point(x=xmax, y=ymin), Point(x=xmax, y=ymax)
