@@ -5,7 +5,6 @@ support for default and constant struct fields, nested structs, arrays of
 structs, better handling for strings and arrays, struct inheritance, and
 convenient syntax for instantiating and using your custom structs.
 """
-
 import sys
 import struct
 import socket
@@ -30,13 +29,13 @@ __all__ = ["CError", "CWarning",
            "underscorize", "hexdump",
            "BYTE_ORDER", "DEFAULT_TIMEOUT"]
 
-class CError(BaseException):
+class CError(ValueError):
     """the only exception class raised directly by protlib"""
 
 class CWarning(UserWarning):
     """the only warning class directly used by protlib (except DeprecationWarning)"""
 
-__version_info__ = (1, 1, 0, "final", 0)
+__version_info__ = (1, 2, 0, "final", 0)
 __version__ = "{0}.{1}.{2}".format(*__version_info__)
 
 BYTE_ORDER = "!"
@@ -55,7 +54,19 @@ def _inherit_docstrings(klass):
             field.__doc__ = parent.__doc__
     return klass
 
-
+class _chained:
+    def __init__(self, s, f):
+        self.s, self.f = s, f
+        self.has_read = ""
+    
+    def read(self, n):
+        pos = len(self.has_read)
+        if pos < len(self.s):
+            data = self.s[pos : pos + n]
+        else:
+            data = self.f.read(n)
+        self.has_read += data       # shlemeil the painter
+        return data
 
 class CType(object):
     """
@@ -99,38 +110,69 @@ class CType(object):
             warn("{0} settings do not include {1}".format(self.__class__.__name__, ", ".join(extra)), CWarning)
         
         self.__dict__.update(settings)
-        if self.length is not None and not isinstance(self, (CString, CArray)):
-            warn("length has no meaning for {0} objects".format(self.__class__.__name__), CWarning)
-        if isinstance(self, CString) and (not isinstance(self.length, int) or self.length <= 0):
-            raise CError("CString objects require a length attribute")
         if self.__class__ is CType:
             raise CError("CType may not be directly instantiated; use a subclass such as CInt, CString, etc")
+        if self.length is not None and not isinstance(self, (CString, CArray)):
+            warn("length has no meaning for {0} objects".format(self.__class__.__name__), CWarning)
+        if isinstance(self, (CString, CArray)):
+            if not isinstance(self.length, (int, basestring)):
+                raise CError("{0} objects require a string or integer length attribute".format(self.__class__.__name__))
+            elif isinstance(self.length, int) and self.length < 0:
+                raise CError("length integer value must be positive")
         
         self.instances.append(self)
     
     @property
-    def struct_format(self):
+    def maybe(self):
+        """either the always or default value as appropriate"""
+        return self.always if self.always is not None else _get_default(self.default)
+    
+    def real_length(self, cstruct):
+        """
+        CArray and CString fields may be given variable lengths by setting
+        their length field to a string representing the name of the other field
+        in the containing CStruct subclass whose value is their length.  This
+        method returns the integer length value of any CType (which is None
+        for types other than CString and CArray), computing it if necessary
+        by extracting the actual value from the given cstruct instance.  This
+        method raises an error if the computed length is negative.
+        """
+        if isinstance(self.length, (int, NoneType)):
+            return self.length
+        
+        if cstruct is None:
+            raise CError("cstruct not provided to resolve variable-length field with length attribute {0!r}".format(self.length))
+        
+        length = getattr(cstruct, self.length, None)
+        if isinstance(length, (CType, NoneType)):
+            raise CError("{0}.{1} not set and is needed for use as a length attribute".format(cstruct.__class__.__name__, self.length))
+        elif not isinstance(length, int):
+            raise CError("{0}.{1} must be an integer for use as a length attribute".format(cstruct.__class__.__name__, self.length))
+        elif length < 0:
+            raise CError("length field {0}.{1} may not be negative".format(cstruct.__class__.__name__, self.length))
+        return length
+    
+    def struct_format(self, cstruct=None):
         """the format string used to represent this CType in the struct module"""
         return {
-            "CChar":   "b",
-            "CUChar":  "B",
-            "CShort":  "h",
-            "CUShort": "H",
-            "CInt":    "i",
-            "CUInt":   "I",
-            "CLong":   "q",
-            "CULong":  "Q",
-            "CFloat":  "f",
-            "CDouble": "d",
-            "CString": "{0}s".format(self.length)
-        }[self.__class__.__name__]
+            CChar:   "b",
+            CUChar:  "B",
+            CShort:  "h",
+            CUShort: "H",
+            CInt:    "i",
+            CUInt:   "I",
+            CLong:   "q",
+            CULong:  "Q",
+            CFloat:  "f",
+            CDouble: "d",
+            CString: "{0}s".format( self.real_length(cstruct) )
+        }[self.__class__]
     
-    @property
-    def sizeof(self):
+    def sizeof(self, cstruct=None):
         """the number of bytes of binary data needed to represent this CType"""
-        return struct.calcsize(BYTE_ORDER + self.struct_format)
+        return struct.calcsize(BYTE_ORDER + self.struct_format(cstruct))
     
-    def parse(self, f):
+    def parse(self, f, cstruct=None):
         r"""
         Accepts either a string or a file and returns a Python object with 
         the appropriate value.  For example, CInt().parse will return a 
@@ -140,12 +182,12 @@ class CType(object):
         
         CString(length=10).parse("foo\x00barbaz") -> "foo"
         """
-        buf = _fileize(f).read(self.sizeof)
-        if len(buf) < self.sizeof:
-            raise CError("{0} requires {1} bytes and was given {2}".format(self.__class__.__name__, self.sizeof, len(buf)))
-        return struct.unpack(BYTE_ORDER + self.struct_format, buf)[0]
+        buf = _fileize(f).read( self.sizeof(cstruct) )
+        if len(buf) < self.sizeof(cstruct):
+            raise CError("{0} requires {1} bytes and was given {2}".format(self.__class__.__name__, self.sizeof(cstruct), len(buf)))
+        return struct.unpack(BYTE_ORDER + self.struct_format(cstruct), buf)[0]
     
-    def serialize(self, val):
+    def serialize(self, val, cstruct=None):
         r"""
         Serializes the given value into binary data using the struct module.
         
@@ -169,7 +211,7 @@ class CType(object):
         try:
             with warnings.catch_warnings():
                 warnings.simplefilter("error", DeprecationWarning)
-                return struct.pack(BYTE_ORDER + self.struct_format, val)
+                return struct.pack(BYTE_ORDER + self.struct_format(cstruct), val)
         except:
             exc_class, exc, tb = sys.exc_info()
             cerror = CError("{0!r} is not serializable as a {1}: {2}".format(val, self.__class__.__name__, exc.args[0]))
@@ -188,13 +230,13 @@ class CDouble(CType): pass
 
 @_inherit_docstrings
 class CString(CType):
-    def parse(self, f):
-        return CType.parse(self, f).split("\x00")[0]
+    def parse(self, f, cstruct=None):
+        return CType.parse(self, f, cstruct).split("\x00")[0]
     
-    def serialize(self, val):
-        if len(val) > self.length:
-            warn("CString has length {0} and was told to serialize a string of length {1}".format(self.length, len(val)), CWarning)
-        return CType.serialize(self, val)
+    def serialize(self, val, cstruct=None):
+        if len(val) > self.real_length(cstruct):
+            warn("CString has length {0} and was told to serialize a string of length {1}".format(self.real_length(cstruct), len(val)), CWarning)
+        return CType.serialize(self, val, cstruct)
 
 @_inherit_docstrings
 class CArray(CType):
@@ -227,41 +269,44 @@ class CArray(CType):
         self.ctype = ctype
         
         for param in ["default","always"]:
-            if param not in params and getattr(self.ctype, param, None) is not None:
+            if isinstance(length, int) and param not in params and getattr(self.ctype, param) is not None:
                 params[param] = [_get_default(getattr(self.ctype, param)) for i in range(length)]
         
         CType.__init__(self, length=length, **params)
         
         for param,value in [("always",self.always), ("default",self.default)]:
-            if value is not None:
+            if value is not None and not isinstance(self.length, basestring):
                 try:
-                    value = _get_default(value) if param=="default" else value
+                    value = _get_default(value) if param == "default" else value
                     self.serialize( _get_converter(self)(value) )
                 except:
                     exc_class, exc, tb = sys.exc_info()
                     cerror = CError("{0!r} is not a valid {1} CArray value: {2}".format(value, param, exc.args[0]))
                     raise CError, cerror, tb
     
-    @property
-    def struct_format(self):
-        return self.ctype.struct_format * self.length
+    def struct_format(self, cstruct=None):
+        return self.ctype.struct_format() * self.real_length(cstruct)
     
-    def parse(self, f):
+    def parse(self, f, cstruct=None):
         f = _fileize(f)
-        return [self.ctype.parse(f) for i in range(self.length)]
+        return [self.ctype.parse(f, cstruct) for i in range(self.real_length(cstruct))]
     
-    def serialize(self, xs):
-        if len(xs) > self.length:
-            warn("CArray has length {0} and was given {1} elements".format(self.length, len(xs)), CWarning)
-            xs = xs[:self.length]
-        elif len(xs) < self.length:
-            if self.default is not None:
-                xs = xs + _get_default(self.default)[len(xs):]    # avoid += to not mutate the original list
-            if len(xs) < self.length:
-                raise CError("CArray has length {0} and was only given {1} elements".format(self.length, len(xs)))
+    def serialize(self, xs, cstruct=None):
+        length = self.real_length(cstruct)
+        if len(xs) > length:
+            warn("CArray has length {0} and was given {1} elements".format(length, len(xs)), CWarning)
+            xs = xs[:length]
+        elif len(xs) < length:
+            if self.maybe is not None or self.ctype.maybe is not None:
+                default = self.maybe or []
+                if self.ctype.maybe is not None:
+                    default += [self.ctype.maybe for i in range(length - len(default))]
+                xs = xs + default[len(xs):]     # avoid += to not mutate the original list
+            if len(xs) < length:
+                raise CError("CArray has length {0} and was only given {1} elements".format(length, len(xs)))
         
         xs = _get_converter(self)(xs)
-        return "".join(self.ctype.serialize(x) for x in xs)
+        return "".join(self.ctype.serialize(x, cstruct) for x in xs)
 
 @_inherit_docstrings
 class CStructType(CType):
@@ -284,7 +329,6 @@ class CStructType(CType):
     protlib users should probably never need to interact with this class
     directly.
     """
-    
     def __init__(self, subclass, **params):
         """
         In addition to the usual keyword arguments accepted by CTypes,
@@ -293,28 +337,34 @@ class CStructType(CType):
         self.subclass = subclass
         CType.__init__(self, **params)
     
-    @property
-    def struct_format(self):
-        return "".join(ctype.struct_format for name,ctype in self.subclass.get_fields())
+    def struct_format(self, cstruct=None):
+        return "".join(ctype.struct_format(cstruct) for name,ctype in self.subclass.get_fields())
     
-    def parse(self, f):
-        buf = _fileize(f).read(self.sizeof)
-        f = StringIO(buf)
-        if len(buf) < self.sizeof:
-            raise CError("{0} requires {1} bytes and was only given {2} ({3!r})".format(self.subclass.__name__, self.sizeof, len(buf), buf))
-        
-        params = {}
+    def parse(self, f, cstruct=None):
+        f = _fileize(f)
+        inst = self.subclass()
         for name,ctype in self.subclass.get_fields():
-            params[name] = ctype.parse(f)
-        return self.subclass(**params)
+            val = ctype.parse(f, cstruct=inst)
+            setattr(inst, name, val)
+        return inst
     
-    def serialize(self, inst):
+    def serialize(self, inst, cstruct=None):
         serialized = ""
         for name,ctype in self.subclass.get_fields():
-            val = getattr(inst, name, None)
-            if val is None or isinstance(val, CType) and not isinstance(val, CStruct):
+            val = inst.__dict__.get(name)
+            if val is None:
+                if isinstance(ctype, CArray):
+                    if ctype.real_length(inst) == 0:
+                        val = []
+                    else:
+                        for maybe in [ctype.default, ctype.always, ctype.ctype.default, ctype.ctype.always]:
+                            if maybe is not None:
+                                val = []
+                elif isinstance(ctype, CString) and ctype.real_length(inst) == 0:
+                    val = ""
+            if val is None:
                 raise CError(name + " not set")
-            serialized += ctype.serialize(val)
+            serialized += ctype.serialize(val, cstruct=inst)
         return serialized
 
 class CStruct(CType):
@@ -347,12 +397,17 @@ class CStruct(CType):
             warn("{0} fields ({1}) do not include {2}".format(self.__class__.__name__, ", ".join(field_names), ", ".join(non_fields)), CWarning)
         
         for name,ctype in fields:
-            maybe = ctype.always if ctype.always is not None else ctype.default
-            if maybe is not None:
-                setattr(self, name, _get_default(maybe))
-        
-        for name,value in values.items():
-            setattr(self, name, value)
+            if ctype.maybe is not None:
+                setattr(self, name, _get_default(ctype.maybe))
+            elif isinstance(ctype, CString) and isinstance(ctype.length, basestring) \
+                    and not isinstance(getattr(self, ctype.length), CType) and ctype.real_length(self) == 0:
+                setattr(self, name, "")
+            elif isinstance(ctype, CArray) and isinstance(ctype.length, basestring) and ctype.ctype.maybe is not None \
+                    and not isinstance(getattr(self, ctype.length), CType):
+                setattr(self, name, [_get_default(ctype.ctype.maybe) for i in range(ctype.real_length(self))])
+            
+            if name in values:
+                setattr(self, name, values[name])   # set after setting default values to detect invalid defaults
     
     @classmethod
     def get_fields(cls):
@@ -369,13 +424,12 @@ class CStruct(CType):
             raise CError("CStruct classmethods may only be called on subclasses of CStruct")
         
         if "_fields" not in cls.__dict__:   # avoid hasattr because of subclasses
-            uninstantiated = [ctype for name,ctype in cls.__dict__.items() 
+            uninstantiated = [ctype for name,ctype in cls.__dict__.items()
                                     if type(ctype) is type and issubclass(ctype,CType)]
             if uninstantiated:
                 raise CError("Use {0}{2} instead of {0} when declaring a field in your {1} struct".format(uninstantiated[0].__name__, cls.__name__, ".get_type()" if issubclass(uninstantiated[0],CStruct) else "()"))
             
-            directly = [cstruct for name,cstruct in cls.__dict__.items() 
-                                if isinstance(cstruct, CStruct)]
+            directly = [cstruct for name,cstruct in cls.__dict__.items() if isinstance(cstruct, CStruct)]
             if directly:
                 raise CError("Use {0}.get_type() instead of {0}() when declaring a field in your {1} struct".format(directly[0].__class__.__name__, cls.__name__))
             
@@ -387,8 +441,7 @@ class CStruct(CType):
                     break
             
             if top is cls:
-                fields = [[name,ctype] for name,ctype in cls.__dict__.items() 
-                                       if isinstance(ctype, CType)]
+                fields = [[name,ctype] for name,ctype in cls.__dict__.items() if isinstance(ctype, CType)]
                 fields.sort(key = lambda pair: CType.instances.index(pair[1]))
                 
                 positions = [(CType.instances.index(ctype),name,ctype) for name,ctype in fields]
@@ -402,10 +455,16 @@ class CStruct(CType):
                     final = getattr(cls, pair[0])
                     if not isinstance(final, CType):
                         raise CError("{0} field overridden by non-CType {1!r}".format(pair[0], final))
-                    elif final.sizeof != pair[1].sizeof:
+                    elif type(final.length) is not type(pair[1].length) \
+                            or not isinstance(final.length, basestring) and final.sizeof() != pair[1].sizeof():
                         raise CError("{0[0]} field of type {0[1]} was overridden by differently-sized type {1}".format(pair, final.__class__.__name__))
                     pair[1] = final
             
+            names = [name for name,ctype in fields]
+            for (i,(name,ctype)) in enumerate(fields):
+                if isinstance(ctype.length, basestring) and ctype.length in names[i+1:]:      # shlemeil the painter
+                    raise CError("{0}.{1} is the length field for {0}.{2} but appears after it in the struct".format(cls.__name__, ctype.length, name))
+            
             cls._fields = fields
         return cls._fields
     
@@ -420,7 +479,8 @@ class CStruct(CType):
                   be created.  This should never be set to True when using
                   this method to include one struct inside another struct, 
                   which is probably the only time that users will ever call
-                  this method.
+                  this method.  In other words, if you're not sure whether
+                  you should include this parameter, then you shouldn't.
         """
         if cls is CStruct:
             raise CError("CStruct classmethods may only be called on subclasses of CStruct")
@@ -439,20 +499,20 @@ class CStruct(CType):
         return cls.get_type(cached=True).parse(f)
     
     @classmethod
-    def sizeof(cls):
+    def sizeof(cls, cstruct=None):
         """
         Returns the number of bytes needed to represent this CStruct as
         packed binary data.
         """
-        return cls.get_type(cached=True).sizeof
+        return cls.get_type(cached=True).sizeof(cstruct)
     
     @classmethod
-    def struct_format(cls):
+    def struct_format(cls, cstruct=None):
         """
         Returns the struct format string used by the struct module to pack
         and unpack this CStruct.
         """
-        return cls.get_type(cached=True).struct_format
+        return cls.get_type(cached=True).struct_format(cstruct)
     
     def serialize(self):
         """
@@ -472,7 +532,7 @@ class CStruct(CType):
         repr(Point(x=5, y=6)) == "Point(x=5, y=6)"
         """
         params = ["{0}={1!r}".format(name, getattr(self,name))
-                  for name,ctype in self.get_fields() if hasattr(self,name)]
+                  for name,ctype in self.get_fields() if name in self.__dict__]
         return "{0}({1})".format(self.__class__.__name__, ", ".join(params))
     
     @property
@@ -516,7 +576,7 @@ class CStruct(CType):
                 raise CError, cerror, tb
             
             try:
-                field.serialize(value)
+                field.serialize(value, cstruct=self)
             except:
                 exc_class, exc, tb = sys.exc_info()
                 cerror = CError("{0!r} is an invalid value for the {1} field {2}.{3}: {4}".format(value, field.__class__.__name__, self.__class__.__name__, name, exc.args[0]))
@@ -538,10 +598,7 @@ def _to_int(x):
     if isinstance(x, float) and x != int(x):
         warn("Loss of precision when converting a float ({0}) to an integer field".format(x), CWarning)
     return int(x)
-
-for ctype in [CShort, CUShort, CInt, CUInt, CLong, CULong]:
-    _converters[ctype] = _to_int
-del ctype
+_converters.update((ctype, _to_int) for ctype in [CShort, CUShort, CInt, CUInt, CLong, CULong])
 
 def _get_converter(field):
     if isinstance(field, CStructType):
@@ -554,8 +611,6 @@ def _get_converter(field):
     else:
         return _converters.get(field.__class__)
 
-
-
 _formatter = Formatter("%(asctime)s: %(message)s")
 
 class _AlsoPrint(logging.Handler):
@@ -612,7 +667,6 @@ class Parser(object):
     class exists to check the data being parsed to see which struct is being
     read with error checking and logging as appropriate.
     """
-    
     def __init__(self, logger=None, module=None):
         """
         Arguments:
@@ -685,17 +739,16 @@ class Parser(object):
                     break
             
             if code == buf:
-                bufsize = cstruct.sizeof()
-                buf += f.read(bufsize - len(buf))
-                self.logger.log_binary(buf, "received")
-                
-                if len(buf) < bufsize:
-                    self.logger.log_error("{0} struct requires {1} bytes, received {2} ({3!r})", cstruct.__name__, bufsize, len(buf), buf)
-                    return
-                else:
-                    inst = cstruct.parse(buf)
+                chained = _chained(buf, f)
+                try:
+                    inst = cstruct.parse(chained)
                     self.logger.log_struct(inst, "received")
                     return inst
+                except CError as ce:
+                    self.logger.log_error("{0} struct received only {1} bytes".format(cstruct.__name__, len(chained.has_read)))
+                    return
+                finally:
+                    self.logger.log_binary(chained.has_read, "received")
         else:
             buf += f.read()
         
@@ -924,7 +977,6 @@ class TCPHandler(ProtHandler, StreamRequestHandler):
     Subclass this class for use with the SocketServer.TCPServer class or any
     of its subclasses, such as protlib.LoggingTCPServer
     """
-    
     def __init__(self, request, client_addr, server):
         """you'll probably never need to instantiate this class directly"""
         ProtHandler.__init__(self, server)
@@ -952,7 +1004,6 @@ class UDPHandler(ProtHandler, DatagramRequestHandler):
     Subclass this class for use with the SocketServer.UDPServer class or any
     of its subclasses, such as protlib.LoggingUDPServer
     """
-    
     def __init__(self, request, client_addr, server):
         """you'll probably never need to instantiate this class directly"""
         ProtHandler.__init__(self, server)
