@@ -11,6 +11,7 @@ from glob import glob
 from time import sleep
 from threading import Thread
 from StringIO import StringIO
+from logging import getLogger, INFO
 
 import unittest
 from unittest import TestCase
@@ -19,6 +20,7 @@ import warnings
 warnings.simplefilter("error", CWarning)
 
 socket.setdefaulttimeout(0.1)
+getLogger().level = INFO
 
 def delete_logs():
     atexit._exithandlers[:] = []    # stop the logging module's exit handler
@@ -45,6 +47,11 @@ class NamedOrigin(NamedPoint):
     code = CShort(always = 0x2332)
     x = CInt(always = 0)
 
+class PointGroup(CStruct):
+    code   = CUChar(always = 255)
+    count  = CShort()
+    points = CArray("count", NamedPoint)
+
 SERVER_ADDR = ("127.0.0.1", 7357)
 CLIENT_ADDR = ("127.0.0.1", 5737)
 
@@ -54,6 +61,11 @@ class TestHandler(object):
     
     def renamed_point(self, rp):
         return "Hello World!\n"
+    
+    def point_group(self, pg):
+        pg.count *= 2
+        pg.points *= 2
+        return pg
 
 class TCPTestHandler(TCPHandler, TestHandler): pass
 class UDPTestHandler(UDPHandler, TestHandler): pass
@@ -172,12 +184,17 @@ class CTypeTests(TestCase):
         self.assertEqual(Point().xy, [5,6])
         
         class Point(CStruct):
-            xy = CArray(2, CInt(default=0))
-        self.assertEqual(Point().xy, [0,0])
+            xy = CArray(2, CInt(default=5))
+        self.assertEqual(Point().xy, [5,5])
         
         class Point(CStruct):
             xy = CArray(2, CInt, default=lambda: [5,6])
         self.assertEqual(Point().xy, [5,6])
+        
+        class Point(CStruct):
+            xy = CArray(2, CInt(default=6), default=[5])
+        self.assertEqual(Point().xy, [5])
+        self.assertEqual([5,6], Point.parse(Point().serialize()).xy)
     
     def test_invalid_arrays(self):
         self.assertRaises(CError, CArray, 10, int)
@@ -262,7 +279,7 @@ class CTypeTests(TestCase):
         self.assertRaises(CWarning, NamedPoint.parse, "!C\x00\x00\x00\x05\x00\x00\x00\x06unnamed\x00\x00\x00\x00\x00\x00\x00\x00")
     
     def test_invalid_structs(self):
-        self.assertRaises(CError, NamedPoint.parse, "not enough data")
+        self.assertRaises(CError, NamedPoint.parse, NP_BUF[:-1])
         
         class Point(CStruct):
             pass
@@ -351,13 +368,160 @@ class CTypeTests(TestCase):
             CArray(2, CInt, default=lambda:[0, 1.2])
             protlib.__warningregistry__.clear()
 
+class VarlengthTests(TestCase):
+    def test_varlength_structs(self):
+        buf = "\xff\x00\x02" + NP_BUF + NP_BUF
+        pg = PointGroup(count=2, points=[NamedPoint(x=5,y=6), NamedPoint(x=5,y=6)])
+        self.assertEqual(pg.serialize(), buf)
+        self.assertEqual(pg, PointGroup.parse(buf))
+    
+    def test_nested_varlength(self):
+        class GeoStuff(CStruct):
+            code    = CShort(always = 255)
+            pg      = PointGroup.get_type()
+            message = CString(length=100, default="hw")
+        
+        buf = "\x00\xff\xff\x00\x02" + NP_BUF + NP_BUF + "hw" + "\x00" * 98
+        pg = PointGroup(count=2, points=[NamedPoint(x=5,y=6), NamedPoint(x=5,y=6)])
+        gs = GeoStuff(pg = pg)
+        self.assertEqual(gs.message, "hw")
+        self.assertEqual(gs.serialize(), buf)
+        self.assertEqual(gs, GeoStuff.parse(buf))
+    
+    def test_multi_varlength(self):
+        class Messages(CStruct):
+            glen     = CChar()
+            greeting = CString(length = "glen")
+            flen     = CChar()
+            farewell = CString(length = "flen")
+            version  = CChar(always = 9)
+        
+        buf = "\x05hello\x07goodbye\x09"
+        mess = Messages(greeting="hello", farewell="goodbye", glen=5, flen=7)
+        self.assertEqual(mess.version, 9)
+        self.assertEqual(mess.serialize(), buf)
+        self.assertEqual(mess, Messages.parse(buf))
+    
+    def test_derived_varlength(self):
+        class AnnoyingSizing(CStruct):
+            size = CShort()
+            arr  = CArray("real_size", CChar)
+            
+            @property
+            def real_size(self):
+                return self.size / 2
+        
+        buf = "\x00\x06\x01\x02\x03"
+        ann = AnnoyingSizing(size=6, arr=[1,2,3])
+        self.assertEqual(ann.serialize(), buf)
+        self.assertEqual(ann, AnnoyingSizing.parse(buf))
+        
+        AnnoyingSizing.real_size = "not an integer"
+        self.assertRaises(CError, AnnoyingSizing.parse, buf)
+    
+    def test_varlength_standalone(self):
+        mock = type("mock", (), {"foo": 15})
+        
+        cs = CString(length = "foo")
+        buf = cs.serialize("Hello World!", cstruct=mock)
+        s = cs.parse(buf, cstruct=mock)
+        self.assertEqual(15, len(buf))
+        self.assertEqual(s, "Hello World!")
+        
+        ca = CArray("foo", CChar(default=0))
+        buf = ca.serialize([1,2,3], cstruct=mock)
+        xs = ca.parse(buf, cstruct=mock)
+        self.assertEqual(15, len(buf))
+        self.assertEqual(xs, [1,2,3] + [0]*12)
+    
+    def test_varlength_attrs(self):
+        mock = type("mock", (), {"foo": 3})
+        pg = PointGroup(count=1, points=[NamedPoint(x=5,y=6)])
+        self.assertEqual(28, PointGroup.sizeof(cstruct = pg))
+        self.assertEqual("Bh" + NamedPoint.struct_format(), PointGroup.struct_format(cstruct = pg))
+        self.assertEqual(12, CArray("foo", CInt).sizeof(cstruct = mock))
+        self.assertEqual("hhh", CArray("foo", CShort).struct_format(cstruct = mock))
+    
+    def test_varlength_missing_length(self):
+        self.assertRaises(CError, CString(length="foo").sizeof)
+        self.assertRaises(CError, CString(length="foo").struct_format)
+        self.assertRaises(CError, PointGroup.sizeof)
+        self.assertRaises(CError, PointGroup.struct_format)
+    
+    def test_varlength_bad_lengths(self):
+        class VarLen(CStruct):
+            size = CChar()
+            arr  = CArray("size", CChar)
+        
+        vl = VarLen(size=2, arr=[1,2])
+        vl.arr.pop()
+        self.assertRaises(CError, vl.serialize)
+        vl.arr.extend([5,6])
+        self.assertRaises(CWarning, vl.serialize)
+        
+        self.assertRaises(CError, VarLen, arr=[1,2])
+        self.assertRaises(CError, VarLen, size=2, arr=[1])
+        self.assertRaises(CError, VarLen, size=2, arr=[1,2,3])
+        self.assertRaises(CError, VarLen.parse, "\x02\x05")
+    
+    def test_varlength_bad_order(self):
+        class BadVarlength(CStruct):
+            arr  = CArray("size", CChar)
+            size = CChar()
+        
+        self.assertRaises(CError, BadVarlength)
+        self.assertRaises(CError, BadVarlength, size=2, arr=[5,6])
+        self.assertRaises(CError, BadVarlength.parse, "\x05\x06\x02")
+    
+    def test_negative_varlength(self):
+        class NegVarLength(CStruct):
+            size = CChar()
+            arr  = CArray("size", CInt(default=0))
+        self.assertRaises(CError, NegVarLength, -1)
+        self.assertRaises(CError, NegVarLength.parse, "\xFF")
+    
+    def test_varlength_matrix(self):
+        for outer_size in [2, "size"]:
+            class Matrix(CStruct):
+                size   = CChar()
+                matrix = CArray(outer_size, CArray("size", CChar))
+            
+            buf = "\x02\x05\x06\x07\x0b"
+            mat = Matrix(size=2, matrix=[[5,6],[7,11]])
+            self.assertEqual(mat.serialize(), buf)
+            self.assertEqual(mat, Matrix.parse(buf))
+    
+    def test_varlength_defaults(self):
+        class DefaultEmpty(CStruct):
+            size   = CChar(default = 0)
+            string = CString(length = "size")
+        self.assertEqual("\x00", DefaultEmpty.parse("\x00").serialize())
+        self.assertEqual("", DefaultEmpty.parse("\x00").string)
+        self.assertEqual("", DefaultEmpty().string)
+        
+        class DefaultEmpty(CStruct):
+            size = CChar(default = 0)
+            arr  = CArray("size", CInt)
+        self.assertEqual("\x00", DefaultEmpty.parse("\x00").serialize())
+        
+        class DefaultConflict(CStruct):
+            size = CChar(default = 2)
+            arr = CArray("size", CInt, default=[])
+        self.assertRaises(CError, DefaultConflict.parse, "\x00")
+        self.assertRaises(CError, DefaultConflict)
+        
+        class DerivedDefault(CStruct):
+            size = CChar(default = 2)
+            arr = CArray("size", CInt(default=5))
+        self.assertEqual(DerivedDefault().arr, [5,5])
+
 class UnderscorizeTests(TestCase):
     def test_camelcase(self):
         self.assertEqual("some_struct",   underscorize("SomeStruct"))
         self.assertEqual("ssn_lookup",    underscorize("SSNLookup"))
         self.assertEqual("rs485_adaptor", underscorize("RS485Adaptor"))
         self.assertEqual("rot13_encoded", underscorize("Rot13Encoded"))
-        self.assertEqual("request_q",     underscorize("request_q"))
+        self.assertEqual("request_q",     underscorize("RequestQ"))
         self.assertEqual("john316",       underscorize("John316"))
     
     def test_already_underscored(self):
@@ -389,8 +553,7 @@ class BadHandlerTests(TestCase):
 
 class ServerTests:
     def setUp(self):
-        self.reset_logs()
-        
+        self.reset_logs()        
         self.server = self.ServerClass(SERVER_ADDR, self.HandlerClass)
         t = Thread(target=self.server.serve_forever)
         t.daemon = True
@@ -422,7 +585,8 @@ class ServerTests:
     
     def test_struct_response(self):
         self.send(self.np)
-        rp = RenamedPoint.parse(self.f)
+        s = self.f.read( RenamedPoint.sizeof() )
+        rp = RenamedPoint.parse(s)
         self.assertEqual(rp, self.rp)
     
     def test_string_response(self):
@@ -432,8 +596,7 @@ class ServerTests:
     def test_too_short(self):
         self.send( self.np.serialize()[:5] )
         sleep(0.2)
-        #import pdb ; pdb.set_trace()
-        self.assertTrue("struct requires" in self.read_log("error"))
+        self.assertTrue("struct received only" in self.read_log("error"))
     
     def test_no_handler(self):
         self.send( NamedOrigin() )
@@ -450,6 +613,19 @@ class ServerTests:
         self.client_teardown()
         self.client_setup()
         self.test_struct_response()
+    
+    def test_varlength(self):
+        pg1 = PointGroup(count=2, points=[NamedPoint(x=5,y=6), NamedPoint(x=7,y=11)])
+        self.send(pg1)
+        pg2 = PointGroup.parse( self.f.read(3 + 4 * NamedPoint.sizeof()) )
+        self.assertEqual(pg2.count, 4)
+        self.assertEqual(pg2.points[:2], pg2.points[2:])
+    
+    def test_varlength_too_short(self):
+        pg = PointGroup(count=2, points=[NamedPoint(x=5,y=6), NamedPoint(x=7,y=11)])
+        self.send( pg.serialize()[:-1] )
+        sleep(0.2)
+        self.assertTrue("struct received only" in self.read_log("error"))
 
 class TCPServerTests(ServerTests, TestCase):
     ServerClass = LoggingTCPServer
@@ -460,7 +636,7 @@ class TCPServerTests(ServerTests, TestCase):
         self.f = self.sock.makefile("r+b", bufsize=0)
     
     def send(self, data):
-        self.f.write(data)
+        self.f.write( str(data) )
         self.f.flush()
 
 class UDPServerTests(ServerTests, TestCase):
